package com.gic.haoban.manage.service.service;

import com.gic.haoban.manage.api.dto.StaffDTO;
import com.gic.haoban.manage.service.entity.TabHaobanStaff;
import com.github.pagehelper.Page;

import java.util.List;

public interface StaffService {

    TabHaobanStaff selectById(String staffId);

    TabHaobanStaff selectByNationcodeAndPhoneNumber(String wxEnterpriseId, String nationcode, String phoneNumber);

    String add(TabHaobanStaff tab);

    Page<TabHaobanStaff> pageStaff(List<String> staffIds, Integer activeFlag,
                                   String keyword,List<String> departmentIds,String wxEnterpriseId,Integer relationFlag);

    void updateByPrimaryKey(TabHaobanStaff tab);

    int delByStaffId(String staffId);

    /**
     * 选择用户id和企业标识
     *
     * @param userId         用户id
     * @param wxEnterpriseId wx企业标识
     * @return {@link TabHaobanStaff }
     * @author mozhu
     * @date 2022-01-13 15:02:30
     */
    TabHaobanStaff selectByUserIdAndEnterpriseId(String userId, String wxEnterpriseId);


    /**
     * 选择电话号码和企业标识
     *
     * @param phoneNumber    电话号码
     * @param wxEnterpriseId wx企业标识
     * @return {@link TabHaobanStaff }
     * @author mozhu
     * @date 2022-01-12 17:19:28
     */
    TabHaobanStaff selectByPhoneNumberAndEnterpriseId(String phoneNumber, String wxEnterpriseId);

    /**
     * 选择userId跟手机号查询数据  userid优先级高
     *
     * @param phoneNumber    电话号码
     * @param wxEnterpriseId wx企业标识
     * @return {@link TabHaobanStaff }
     * @author mozhu
     * @date 2022-01-12 17:19:28
     */
    TabHaobanStaff selectByPhoneNumberOrUserIdAndEnterpriseId(String phoneNumber,String userId, String wxEnterpriseId);

    List<StaffDTO> listByUserIdsAndWxEnterpriseId(List<String> userIds, String wxEnterpriseId);

    /**
     * 清楚微信信息
     *
     * @param wxEnterpriseId
     * @return
     */
    int cleanStaff(String wxEnterpriseId, List<String> staffIds);

    TabHaobanStaff getWxStaffByOne(String wxEnterpriseId);

    /**
     * 获取需要删除的staff列表
     *
     * @param wxEnterpriseId
     * @param staffIds
     * @return
     */
    List<String> listDelUserStaffId(String wxEnterpriseId, List<String> staffIds);

    /**
     * 获取无状态的staff列表
     *
     * @param wxEnterpriseId
     * @param staffIds
     * @return
     */
    List<TabHaobanStaff> listNoStatusByStaffIds(String wxEnterpriseId, List<String> staffIds);

    /**
     * 通过开放用户id和wx企业id列表
     *
     * @param openUserIds    开放的用户id
     * @param wxEnterpriseId wx企业标识
     * @return {@link List<StaffDTO> }
     * @author mozhu
     * @date 2021-12-21 16:32:50
     */
    List<StaffDTO> listByOpenUserIdsAndWxEnterpriseId(List<String> openUserIds, String wxEnterpriseId);

    /**
     * 更新开放用户id,用户id
     *
     * @param wxEnterpriseId wx企业标识
     * @param wxUserId       wx用户id
     * @param wxOpenUseId    wx开放使用id
     * @return int
     * @author mozhu
     * @date 2021-12-21 17:19:33
     */
    int updateOpenUserIdsByUserId(String wxEnterpriseId, String wxUserId, String wxOpenUseId);

    /**
     * 删除同wxUserId的其他staff
     *
     * @param wxUserId       微信用户id
     * @param staffId        导购id
     * @param wxEnterpriseId wx企业标识
     */
    void delOtherStaffByWxUserId(String wxUserId, String staffId, String wxEnterpriseId);
    
}
